package biblioteca;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;

public class Main extends JFrame {

	private ArrayList<Libro> elenco;

	private JPanel contentPane;
	private JTextArea txtElenco;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main frame = new Main();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 */
	public Main() {
		elenco = new ArrayList<>(100);

		setTitle("Biblioteca");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 450, 300);

		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);

		JMenu mnFile = new JMenu("File");
		menuBar.add(mnFile);

		JMenuItem mntmNuovo = new JMenuItem("Nuovo");
		mntmNuovo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				nuovaBiblioteca();
			}
		});
		mntmNuovo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		mnFile.add(mntmNuovo);

		JMenuItem mntmApri = new JMenuItem("Apri");
		mntmApri.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				apriBiblioteca();
			}
		});
		mntmApri.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));
		mnFile.add(mntmApri);

		JMenuItem mntmSalva = new JMenuItem("Salva");
		mntmSalva.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				salvaBiblioteca();
			}
		});
		mntmSalva.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		mnFile.add(mntmSalva);

		JMenu mnModifica = new JMenu("Modifica");
		menuBar.add(mnModifica);

		JMenuItem mntmAggiungiLibro = new JMenuItem("Aggiungi libro");
		mntmAggiungiLibro.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				aggiungiLibro();
			}
		});
		mnModifica.add(mntmAggiungiLibro);

		JMenuItem mntmEliminaLibro = new JMenuItem("Elimina libro");
		mntmEliminaLibro.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				eliminaLibro();
			}
		});
		mnModifica.add(mntmEliminaLibro);

		JMenuItem mntmVisualizzaLibri = new JMenuItem("Visualizza libri");
		mntmVisualizzaLibri.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				visualizzaLibri();
			}
		});
		mnModifica.add(mntmVisualizzaLibri);

		JMenu mnRicerca = new JMenu("Ricerca");
		menuBar.add(mnRicerca);

		JMenuItem mntmRicercaPerAutore = new JMenuItem("Ricerca per autore");
		mntmRicercaPerAutore.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ricercaPerAutore();
			}
		});
		mnRicerca.add(mntmRicercaPerAutore);

		JMenuItem mntmRicercaPerTitolo = new JMenuItem("Ricerca per titolo");
		mnRicerca.add(mntmRicercaPerTitolo);

		JMenu mnNewMenu = new JMenu("?");
		menuBar.add(mnNewMenu);

		JMenuItem mntmInformazioniSu = new JMenuItem("Informazioni su...");
		mntmInformazioniSu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new InformazioniSu().setVisible(true);
			}
		});
		mnNewMenu.add(mntmInformazioniSu);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);

		JScrollPane scrollPane = new JScrollPane();
		contentPane.add(scrollPane, BorderLayout.CENTER);

		txtElenco = new JTextArea();
		txtElenco.setEditable(false);
		scrollPane.setViewportView(txtElenco);
	}

	/**
	 * Metodi associati al menu File
	 */
	
	private void nuovaBiblioteca() {
		int risposta = JOptionPane.showConfirmDialog(contentPane, "Vuoi veramente cancellare l'archivio in memoria?",
				"Avviso", JOptionPane.OK_CANCEL_OPTION);
		if (risposta == JOptionPane.OK_OPTION)
			elenco.clear();

		visualizzaLibri();
	}

	private void apriBiblioteca() {
		elenco.clear();

		try {
			FileReader fr = new FileReader("Biblioteca.csv");
			BufferedReader br = new BufferedReader(fr);

			String linea;
			while ((linea = br.readLine()) != null) {
				String[] dati = linea.split(";");
				Libro libro = new Libro();
				libro.setAutore(dati[0]);
				libro.setTitolo(dati[1]);
				libro.setEditore(dati[2]);
				libro.setPagine(Integer.parseInt(dati[3]));
				libro.setPrezzo(Double.parseDouble(dati[4]));
				libro.setIsbn(dati[5]);
				elenco.add(libro);
			}

			br.close();
			fr.close();
			JOptionPane.showMessageDialog(contentPane, "Caricamento completato", "Informazioni",
					JOptionPane.INFORMATION_MESSAGE);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(contentPane, "Errore lettura file:\n" + e.toString(), "Errore",
					JOptionPane.ERROR_MESSAGE);
		}

		visualizzaLibri();
	}

	private void salvaBiblioteca() {
		try {
			FileWriter fw = new FileWriter("Biblioteca.csv");
			BufferedWriter bw = new BufferedWriter(fw);

			for (int i = 0; i < elenco.size(); i++) {
				Libro libro = elenco.get(i);
				bw.write(libro.toFile());
				bw.newLine();
			}

			bw.close();
			fw.close();
			JOptionPane.showMessageDialog(contentPane, "Salvataggio completato", "Informazioni",
					JOptionPane.INFORMATION_MESSAGE);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(contentPane, "Errore scrittura file:\n" + e.toString(), "Errore",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Metodi associati al menu Modifica
	 */
	
	private void aggiungiLibro() {
		JInputLibro inputLibro = new JInputLibro();
		if (inputLibro.showDialog() == JInputLibro.OK_OPTION)
			elenco.add(inputLibro.getLibro());

		visualizzaLibri();
	}

	private void eliminaLibro() {
		int titolo = -1;

		while (titolo == -1)
			titolo = Integer.parseInt(JOptionPane.showInputDialog(contentPane,
					"Inserisci il numero del titolo da eliminare", "Inserimento", JOptionPane.QUESTION_MESSAGE));

		elenco.remove(titolo - 1);

		visualizzaLibri();
	}

	private void visualizzaLibri() {
		txtElenco.setText("");

		for (int i = 0; i < elenco.size(); i++) {
			Libro libro = elenco.get(i);
			txtElenco.append(i + 1 + ".\n" + libro.toString() + "\n");
		}
	}

	/**
	 * Metodi associati al menu Ricerca
	 */
	
	private void ricercaPerAutore() {
		String autore = null;

		while (autore == null)
			autore = JOptionPane.showInputDialog(contentPane, "Inserisci l'autore:", "Ricerca per autore",
					JOptionPane.QUESTION_MESSAGE);

		txtElenco.setText("Ricerca per autore = " + autore + "\n\n");
		for (int i = 0; i < elenco.size(); i++) {
			Libro libro = elenco.get(i);
			if (libro.getAutore().equals(autore))
				txtElenco.append(i + 1 + ". " + libro.toString() + "\n");
		}
	}

}
